@@ -6,7 +6,6 @@ import sys
 from contextlib import AsyncExitStack
 from contextlib import suppress
 from functools import wraps
-from os import environ as env
 from pathlib import Path
 from typing import Any
 from typing import Awaitable
@@ -18,11 +17,10 @@ import asyncclick as click
 
 from dipdup import __spec_version__
 from dipdup import __version__
+from dipdup import env
 from dipdup import spec_reindex_mapping
 from dipdup import spec_version_mapping
 from dipdup.utils.sys import IGNORE_CONFIG_CMDS
-from dipdup.utils.sys import is_in_ci
-from dipdup.utils.sys import is_in_tests
 from dipdup.utils.sys import set_up_logging
 from dipdup.utils.sys import set_up_process
 
@@ -165,12 +163,13 @@ async def cli(ctx: click.Context, config: list[str], env_file: list[str]) -> Non
     _config = DipDupConfig.load(config_paths)
     _config.set_up_logging()
 
-    # NOTE: Imports will be loaded later if needed
-    _config.initialize(skip_imports=True)
     init_sentry(_config)
 
+    # NOTE: Imports will be loaded later if needed
+    _config.initialize(skip_imports=True)
+
     # NOTE: Fire and forget, do not block instant commands
-    if not any((_config.advanced.skip_version_check, is_in_tests(), is_in_ci())):
+    if not any((_config.advanced.skip_version_check, env.TEST, env.CI)):
         asyncio.ensure_future(_check_version())
 
     # NOTE: Avoid import errors if project package is incomplete
@@ -508,12 +507,13 @@ async def schema_export(ctx: click.Context) -> None:
     config: DipDupConfig = ctx.obj.config
     url = config.database.connection_string
     models = f'{config.package}.models'
+    package_path = env.get_package_path(config.package)
 
     async with tortoise_wrapper(url, models):
         conn = get_connection()
         output = get_schema_sql(conn, False) + '\n'
         dipdup_sql_path = Path(__file__).parent / 'sql' / 'on_reindex'
-        project_sql_path = Path(config.package_path) / 'sql' / 'on_reindex'
+        project_sql_path = package_path / 'sql' / 'on_reindex'
 
         for sql_path in (dipdup_sql_path, project_sql_path):
             for file in iter_files(sql_path):
