@@ -19,6 +19,7 @@ from typing import cast
 
 import orjson as json
 
+from dipdup import env
 from dipdup.config import BigMapIndexConfig
 from dipdup.config import CallbackMixin
 from dipdup.config import ContractConfig
@@ -31,6 +32,7 @@ from dipdup.config import OperationHandlerOriginationPatternConfig as Originatio
 from dipdup.config import OperationHandlerPatternConfigU as PatternConfigU
 from dipdup.config import OperationHandlerTransactionPatternConfig as TransactionPatternConfig
 from dipdup.config import OperationIndexConfig
+from dipdup.config import OperationUnfilteredIndexConfig
 from dipdup.config import TokenTransferIndexConfig
 from dipdup.config import TzktDatasourceConfig
 from dipdup.config import UnknownEventHandlerConfig
@@ -100,6 +102,7 @@ class ProjectPaths:
         self.hooks = root / 'hooks'
         self.sql = root / 'sql'
         self.graphql = root / 'graphql'
+        self.hasura = root / 'hasura'
 
     def create_package(self) -> None:
         """Create Python package skeleton if not exists"""
@@ -112,6 +115,7 @@ class ProjectPaths:
 
         touch(self.sql / KEEP_MARKER)
         touch(self.graphql / KEEP_MARKER)
+        touch(self.hasura / KEEP_MARKER)
 
         if not self.models.is_file():
             template = load_template('templates', f'{MODELS_MODULE}.j2')
@@ -127,7 +131,7 @@ class CodeGenerator:
         self._config = config
         self._datasources = datasources
         self._schemas: dict[TzktDatasourceConfig, dict[str, dict[str, Any]]] = {}
-        self._pkg = ProjectPaths(Path(config.package_path))
+        self._pkg = ProjectPaths(env.get_package_path(config.package))
 
     def create_package(self) -> None:
         self._pkg.create_package()
@@ -285,6 +289,8 @@ class CodeGenerator:
                 pass
             elif isinstance(index_config, TokenTransferIndexConfig):
                 pass
+            elif isinstance(index_config, OperationUnfilteredIndexConfig):
+                pass
             elif isinstance(index_config, IndexTemplateConfig):
                 raise ConfigInitializationException
             else:
@@ -389,6 +395,10 @@ class CodeGenerator:
         for index_config in self._config.indexes.values():
             if isinstance(index_config, IndexTemplateConfig):
                 continue
+            if isinstance(index_config, OperationUnfilteredIndexConfig):
+                await self._generate_callback(index_config.handler_config)
+                continue
+
             for handler_config in index_config.handlers:
                 await self._generate_callback(handler_config)
 
