@@ -49,7 +49,7 @@ class Error(ABC, FrameworkException):
     def __str__(self) -> str:
         if not self.__doc__:
             raise NotImplementedError(f'{self.__class__.__name__} has no docstring')
-        return self.__doc__
+        return self.__doc__ + ' -> ' + ' '.join(self.args)
 
     def help(self) -> str:
         """Return a string containing a help message for this error."""
