@@ -1,14 +1,10 @@
-import logging
 from abc import ABC
 from abc import abstractmethod
 from collections import deque
 from contextlib import ExitStack
 from typing import Any
 from typing import Generic
-from typing import Optional
-from typing import Set
 from typing import TypeVar
-from typing import Union
 from typing import cast
 
 import dipdup.models as models
@@ -20,35 +16,13 @@ from dipdup.datasources.datasource import IndexDatasource
 from dipdup.enums import IndexStatus
 from dipdup.enums import MessageType
 from dipdup.exceptions import FrameworkException
-from dipdup.models import BigMapData
-from dipdup.models import EventData
-from dipdup.models import OperationData
-from dipdup.models import Origination
-from dipdup.models import TokenTransferData
-from dipdup.models import Transaction
 from dipdup.prometheus import Metrics
 from dipdup.utils import FormattedLogger
 
-_logger = logging.getLogger(__name__)
-
 IndexConfigT = TypeVar('IndexConfigT', bound=ResolvedIndexConfigU)
 IndexQueueItemT = TypeVar('IndexQueueItemT', bound=Any)
 IndexDatasourceT = TypeVar('IndexDatasourceT', bound=IndexDatasource)
 
-OperationHandlerArgumentT = Optional[Union[Transaction, Origination, OperationData]]
-
-
-# TODO: Not used in some indexes
-def extract_level(
-    message: tuple[OperationData | BigMapData | TokenTransferData | EventData, ...],
-) -> int:
-    """Safely extract level from raw messages batch"""
-    # TODO: Skip conditionally
-    batch_levels = {(i.level, i.__class__) for i in message}
-    if len(batch_levels) != 1:
-        raise FrameworkException(f'Items in data batch have different levels: {batch_levels}')
-    return batch_levels.pop()[0]
-
 
 class Index(ABC, Generic[IndexConfigT, IndexQueueItemT, IndexDatasourceT]):
     """Base class for index implementations
@@ -74,7 +48,7 @@ class Index(ABC, Generic[IndexConfigT, IndexQueueItemT, IndexDatasourceT]):
         self._queue: deque[IndexQueueItemT] = deque()
 
         self._logger = FormattedLogger('dipdup.index', fmt=f'{config.name}: ' + '{}')
-        self._state: Optional[models.Index] = None
+        self._state: models.Index | None = None
 
     def push_realtime_message(self, message: IndexQueueItemT) -> None:
         """Push message to the queue"""
@@ -124,9 +98,9 @@ class Index(ABC, Generic[IndexConfigT, IndexQueueItemT, IndexDatasourceT]):
             raise FrameworkException('Call `set_sync_level` before starting `IndexDispatcher`')
         # NOTE: Multiple sync levels means index with new subscriptions was added in runtime.
         # NOTE: Choose the highest level; outdated realtime messages will be dropped from the queue anyway.
-        return max(cast(Set[int], sync_levels))
+        return max(cast(set[int], sync_levels))
 
-    async def initialize_state(self, state: Optional[models.Index] = None) -> None:
+    async def initialize_state(self, state: models.Index | None = None) -> None:
         if self._state:
             raise FrameworkException('Index state is already initialized')
 
@@ -151,17 +125,21 @@ class Index(ABC, Generic[IndexConfigT, IndexQueueItemT, IndexDatasourceT]):
         )
 
     async def process(self) -> bool:
+        if self.state.status == IndexStatus.ONESHOT:
+            raise FrameworkException('Index is in oneshot state and cannot be processed')
+
         if self.name in rolled_back_indexes:
             await self.state.refresh_from_db(('level',))
             rolled_back_indexes.remove(self.name)
 
-        if not isinstance(self._config, HeadIndexConfig) and self._config.last_level:
-            head_level = self._config.last_level
+        last_level = self._config.last_level
+        if last_level and not isinstance(self._config, HeadIndexConfig):
             with ExitStack() as stack:
                 if Metrics.enabled:
                     stack.enter_context(Metrics.measure_total_sync_duration())
-                await self._synchronize(head_level)
-            await self.state.update_status(IndexStatus.ONESHOT, head_level)
+                await self._synchronize(last_level)
+                await self._enter_disabled_state(last_level)
+                return True
 
         index_level = self.state.level
         sync_level = self.get_sync_level()
@@ -174,21 +152,18 @@ class Index(ABC, Generic[IndexConfigT, IndexQueueItemT, IndexDatasourceT]):
                 if Metrics.enabled:
                     stack.enter_context(Metrics.measure_total_sync_duration())
                 await self._synchronize(sync_level)
+                return True
 
-        elif self._queue:
+        if self._queue:
             with ExitStack() as stack:
                 if Metrics.enabled:
                     stack.enter_context(Metrics.measure_total_realtime_duration())
                 await self._process_queue()
-        else:
-            return False
-        return True
+                return True
 
-    async def _enter_sync_state(self, head_level: int) -> Optional[int]:
-        # NOTE: Final state for indexes with `last_level`
-        if self.state.status == IndexStatus.ONESHOT:
-            return None
+        return False
 
+    async def _enter_sync_state(self, head_level: int) -> int | None:
         index_level = self.state.level
 
         if index_level == head_level:
@@ -197,11 +172,27 @@ class Index(ABC, Generic[IndexConfigT, IndexQueueItemT, IndexDatasourceT]):
             raise FrameworkException(f'Attempt to synchronize index from level {index_level} to level {head_level}')
 
         self._logger.info('Synchronizing index to level %s', head_level)
-        await self.state.update_status(status=IndexStatus.SYNCING, level=index_level)
+        await self._update_state(status=IndexStatus.SYNCING, level=index_level)
         return index_level
 
     async def _exit_sync_state(self, head_level: int) -> None:
         self._logger.info('Index is synchronized to level %s', head_level)
         if Metrics.enabled:
             Metrics.set_levels_to_sync(self._config.name, 0)
-        await self.state.update_status(status=IndexStatus.REALTIME, level=head_level)
+        await self._update_state(status=IndexStatus.REALTIME, level=head_level)
+
+    async def _enter_disabled_state(self, last_level: int) -> None:
+        self._logger.info('Index is synchronized to level %s', last_level)
+        if Metrics.enabled:
+            Metrics.set_levels_to_sync(self._config.name, 0)
+        await self._update_state(status=IndexStatus.ONESHOT, level=last_level)
+
+    async def _update_state(
+        self,
+        status: IndexStatus | None = None,
+        level: int | None = None,
+    ) -> None:
+        state = self.state
+        state.status = status or state.status
+        state.level = level or state.level
+        await state.save()
