@@ -17,10 +17,9 @@ from dipdup.utils.codegen import write
 _logger = logging.getLogger('dipdup.project')
 
 
-# FIXME
 DEMO_PROJECTS = (
     ('demo_domains', 'Tezos Domains name service'),
-    ('demo_big_maps', 'Indexing a single big map'),
+    ('demo_big_maps', 'Indexing specific big maps'),
     ('demo_events', 'Processing contract events'),
     ('demo_head', 'Processing head block metadata'),
     ('demo_nft_marketplace', 'hic at nunc NFT marketplace'),
@@ -30,6 +29,7 @@ DEMO_PROJECTS = (
     ('demo_token', 'TzBTC FA1.2 token operations'),
     ('demo_token_transfers', 'TzBTC FA1.2 token transfers'),
     ('demo_auction', 'TzColors NFT marketplace'),
+    ('demo_raw', 'Process raw operations without filtering and strict typing (experimental)'),
     ('blank', 'Empty config for a fresh start'),
     # TODO: demo_jobs
     # TODO: demo_backup
@@ -285,12 +285,12 @@ class BaseProject(Project):
             ),
             default=0,
             choices=(
-                'hasura/graphql-engine:v2.15.2',
-                'hasura/graphql-engine:v2.15.2',
+                'hasura/graphql-engine:v2.19.0',
+                'hasura/graphql-engine:v2.19.0-beta.1',
             ),
             comments=(
-                f'tested with DipDup {major_version}',
-                'latest',
+                'stable',
+                'beta',
             ),
         ),
         NotifyQuestion(
